@@ -15,32 +15,37 @@
 """
 from collections import defaultdict
 
+
 try:
     from zope.event import notify
 except ImportError: # pragma: no cover
     def notify(*arg, **kw): pass
 
-from zope.interface.interfaces import ISpecification
+from zope.interface.adapter import AdapterRegistry
+from zope.interface.declarations import implementedBy
+from zope.interface.declarations import implementer
+from zope.interface.declarations import implementer_only
+from zope.interface.declarations import providedBy
+from zope.interface.interface import Interface
 from zope.interface.interfaces import ComponentLookupError
 from zope.interface.interfaces import IAdapterRegistration
 from zope.interface.interfaces import IComponents
 from zope.interface.interfaces import IHandlerRegistration
+from zope.interface.interfaces import ISpecification
 from zope.interface.interfaces import ISubscriptionAdapterRegistration
 from zope.interface.interfaces import IUtilityRegistration
 from zope.interface.interfaces import Registered
 from zope.interface.interfaces import Unregistered
 
-from zope.interface.interface import Interface
-from zope.interface.declarations import implementedBy
-from zope.interface.declarations import implementer
-from zope.interface.declarations import implementer_only
-from zope.interface.declarations import providedBy
-from zope.interface.adapter import AdapterRegistry
-from zope.interface._compat import CLASS_TYPES
-from zope.interface._compat import STRING_TYPES
 
+__all__ = [
+    # Components is public API, but
+    # the *Registration classes are just implementations
+    # of public interfaces.
+    'Components',
+]
 
-class _UnhashableComponentCounter(object):
+class _UnhashableComponentCounter:
     # defaultdict(int)-like object for unhashable components
 
     def __init__(self, otherdict):
@@ -70,7 +75,7 @@ class _UnhashableComponentCounter(object):
 def _defaultdict_int():
     return defaultdict(int)
 
-class _UtilityRegistrations(object):
+class _UtilityRegistrations:
 
     def __init__(self, utilities, utility_registrations):
         # {provided -> {component: count}}
@@ -138,14 +143,14 @@ class _UtilityRegistrations(object):
 
 
 @implementer(IComponents)
-class Components(object):
+class Components:
 
     _v_utility_registrations_cache = None
 
     def __init__(self, name='', bases=()):
         # __init__ is used for test cleanup as well as initialization.
         # XXX add a separate API for test cleanup.
-        assert isinstance(name, STRING_TYPES)
+        assert isinstance(name, str)
         self.__name__ = name
         self._init_registries()
         self._init_registrations()
@@ -153,14 +158,14 @@ class Components(object):
         self._v_utility_registrations_cache = None
 
     def __repr__(self):
-        return "<%s %s>" % (self.__class__.__name__, self.__name__)
+        return "<{} {}>".format(self.__class__.__name__, self.__name__)
 
     def __reduce__(self):
         # Mimic what a persistent.Persistent object does and elide
         # _v_ attributes so that they don't get saved in ZODB.
         # This allows us to store things that cannot be pickled in such
         # attributes.
-        reduction = super(Components, self).__reduce__()
+        reduction = super().__reduce__()
         # (callable, args, state, listiter, dictiter)
         # We assume the state is always a dict; the last three items
         # are technically optional and can be missing or None.
@@ -212,8 +217,8 @@ class Components(object):
         lambda self, bases: self._setBases(bases),
         )
 
-    def registerUtility(self, component=None, provided=None, name=u'',
-                        info=u'', event=True, factory=None):
+    def registerUtility(self, component=None, provided=None, name='',
+                        info='', event=True, factory=None):
         if factory:
             if component:
                 raise TypeError("Can't specify factory and component.")
@@ -222,7 +227,7 @@ class Components(object):
         if provided is None:
             provided = _getUtilityProvided(component)
 
-        if name == u'':
+        if name == '':
             name = _getName(component)
 
         reg = self._utility_registrations.get((provided, name))
@@ -241,7 +246,7 @@ class Components(object):
                                     factory)
                 ))
 
-    def unregisterUtility(self, component=None, provided=None, name=u'',
+    def unregisterUtility(self, component=None, provided=None, name='',
                           factory=None):
         if factory:
             if component:
@@ -277,28 +282,27 @@ class Components(object):
              ) in iter(self._utility_registrations.items()):
             yield UtilityRegistration(self, provided, name, *data)
 
-    def queryUtility(self, provided, name=u'', default=None):
+    def queryUtility(self, provided, name='', default=None):
         return self.utilities.lookup((), provided, name, default)
 
-    def getUtility(self, provided, name=u''):
+    def getUtility(self, provided, name=''):
         utility = self.utilities.lookup((), provided, name)
         if utility is None:
             raise ComponentLookupError(provided, name)
         return utility
 
     def getUtilitiesFor(self, interface):
-        for name, utility in self.utilities.lookupAll((), interface):
-            yield name, utility
+        yield from self.utilities.lookupAll((), interface)
 
     def getAllUtilitiesRegisteredFor(self, interface):
         return self.utilities.subscriptions((), interface)
 
     def registerAdapter(self, factory, required=None, provided=None,
-                        name=u'', info=u'', event=True):
+                        name='', info='', event=True):
         if provided is None:
             provided = _getAdapterProvided(factory)
         required = _getAdapterRequired(factory, required)
-        if name == u'':
+        if name == '':
             name = _getName(factory)
         self._adapter_registrations[(required, provided, name)
                                     ] = factory, info
@@ -312,7 +316,7 @@ class Components(object):
 
 
     def unregisterAdapter(self, factory=None,
-                          required=None, provided=None, name=u'',
+                          required=None, provided=None, name='',
                           ):
         if provided is None:
             if factory is None:
@@ -344,21 +348,21 @@ class Components(object):
             yield AdapterRegistration(self, required, provided, name,
                                       component, info)
 
-    def queryAdapter(self, object, interface, name=u'', default=None):
+    def queryAdapter(self, object, interface, name='', default=None):
         return self.adapters.queryAdapter(object, interface, name, default)
 
-    def getAdapter(self, object, interface, name=u''):
+    def getAdapter(self, object, interface, name=''):
         adapter = self.adapters.queryAdapter(object, interface, name)
         if adapter is None:
             raise ComponentLookupError(object, interface, name)
         return adapter
 
-    def queryMultiAdapter(self, objects, interface, name=u'',
+    def queryMultiAdapter(self, objects, interface, name='',
                           default=None):
         return self.adapters.queryMultiAdapter(
             objects, interface, name, default)
 
-    def getMultiAdapter(self, objects, interface, name=u''):
+    def getMultiAdapter(self, objects, interface, name=''):
         adapter = self.adapters.queryMultiAdapter(objects, interface, name)
         if adapter is None:
             raise ComponentLookupError(objects, interface, name)
@@ -374,7 +378,7 @@ class Components(object):
 
     def registerSubscriptionAdapter(self,
                                     factory, required=None, provided=None,
-                                    name=u'', info=u'',
+                                    name='', info='',
                                     event=True):
         if name:
             raise TypeError("Named subscribers are not yet supported")
@@ -397,7 +401,7 @@ class Components(object):
             yield SubscriptionRegistration(self, *data)
 
     def unregisterSubscriptionAdapter(self, factory=None,
-                          required=None, provided=None, name=u'',
+                          required=None, provided=None, name='',
                           ):
         if name:
             raise TypeError("Named subscribers are not yet supported")
@@ -443,7 +447,7 @@ class Components(object):
 
     def registerHandler(self,
                         factory, required=None,
-                        name=u'', info=u'',
+                        name='', info='',
                         event=True):
         if name:
             raise TypeError("Named handlers are not yet supported")
@@ -462,7 +466,7 @@ class Components(object):
         for data in self._handler_registrations:
             yield HandlerRegistration(self, *data)
 
-    def unregisterHandler(self, factory=None, required=None, name=u''):
+    def unregisterHandler(self, factory=None, required=None, name=''):
         if name:
             raise TypeError("Named subscribers are not yet supported")
 
@@ -499,12 +503,78 @@ class Components(object):
     def handle(self, *objects):
         self.adapters.subscribers(objects, None)
 
+    def rebuildUtilityRegistryFromLocalCache(self, rebuild=False):
+        """
+        Emergency maintenance method to rebuild the ``.utilities``
+        registry from the local copy maintained in this object, or
+        detect the need to do so.
+
+        Most users will never need to call this, but it can be helpful
+        in the event of suspected corruption.
+
+        By default, this method only checks for corruption. To make it
+        actually rebuild the registry, pass `True` for *rebuild*.
+
+        :param bool rebuild: If set to `True` (not the default),
+           this method will actually register and subscribe utilities
+           in the registry as needed to synchronize with the local cache.
+
+        :return: A dictionary that's meant as diagnostic data. The keys
+           and values may change over time. When called with a false *rebuild*,
+           the keys ``"needed_registered"`` and ``"needed_subscribed"`` will be
+           non-zero if any corruption was detected, but that will not be corrected.
+
+        .. versionadded:: 5.3.0
+        """
+        regs = dict(self._utility_registrations)
+        utils = self.utilities
+        needed_registered = 0
+        did_not_register = 0
+        needed_subscribed = 0
+        did_not_subscribe = 0
+
+
+        # Avoid the expensive change process during this; we'll call
+        # it once at the end if needed.
+        assert 'changed' not in utils.__dict__
+        utils.changed = lambda _: None
+
+        if rebuild:
+            register = utils.register
+            subscribe = utils.subscribe
+        else:
+            register = subscribe = lambda *args: None
+
+        try:
+            for (provided, name), (value, _info, _factory) in regs.items():
+                if utils.registered((), provided, name) != value:
+                    register((), provided, name, value)
+                    needed_registered += 1
+                else:
+                    did_not_register += 1
+
+                if utils.subscribed((), provided, value) is None:
+                    needed_subscribed += 1
+                    subscribe((), provided, value)
+                else:
+                    did_not_subscribe += 1
+        finally:
+            del utils.changed
+            if rebuild and (needed_subscribed or needed_registered):
+                utils.changed(utils)
+
+        return {
+            'needed_registered': needed_registered,
+            'did_not_register': did_not_register,
+            'needed_subscribed': needed_subscribed,
+            'did_not_subscribe': did_not_subscribe
+        }
 
 def _getName(component):
     try:
         return component.__component_name__
     except AttributeError:
-        return u''
+        return ''
 
 def _getUtilityProvided(component):
     provided = list(providedBy(component))
@@ -540,18 +610,18 @@ def _getAdapterRequired(factory, required):
         if r is None:
             r = Interface
         elif not ISpecification.providedBy(r):
-            if isinstance(r, CLASS_TYPES):
+            if isinstance(r, type):
                 r = implementedBy(r)
             else:
                 raise TypeError("Required specification must be a "
-                                "specification or class."
+                                "specification or class, not %r" % type(r)
                                 )
         result.append(r)
     return tuple(result)
 
 
 @implementer(IUtilityRegistration)
-class UtilityRegistration(object):
+class UtilityRegistration:
 
     def __init__(self, registry, provided, name, component, doc, factory=None):
         (self.registry, self.provided, self.name, self.component, self.info,
@@ -559,7 +629,7 @@ class UtilityRegistration(object):
          ) = registry, provided, name, component, doc, factory
 
     def __repr__(self):
-        return '%s(%r, %s, %r, %s, %r, %r)' % (
+        return '{}({!r}, {}, {!r}, {}, {!r}, {!r})'.format(
                 self.__class__.__name__,
                 self.registry,
                 getattr(self.provided, '__name__', None), self.name,
@@ -589,7 +659,7 @@ class UtilityRegistration(object):
         return repr(self) >= repr(other)
 
 @implementer(IAdapterRegistration)
-class AdapterRegistration(object):
+class AdapterRegistration:
 
     def __init__(self, registry, required, provided, name, component, doc):
         (self.registry, self.required, self.provided, self.name,
@@ -597,7 +667,7 @@ class AdapterRegistration(object):
          ) = registry, required, provided, name, component, doc
 
     def __repr__(self):
-        return '%s(%r, %s, %s, %r, %s, %r)' % (
+        return '{}({!r}, {}, {}, {!r}, {}, {!r})'.format(
             self.__class__.__name__,
             self.registry,
             '[' + ", ".join([r.__name__ for r in self.required]) + ']',
@@ -645,7 +715,7 @@ class HandlerRegistration(AdapterRegistration):
     provided = None
 
     def __repr__(self):
-        return '%s(%r, %s, %r, %s, %r)' % (
+        return '{}({!r}, {}, {!r}, {}, {!r})'.format(
             self.__class__.__name__,
             self.registry,
             '[' + ", ".join([r.__name__ for r in self.required]) + ']',
